package redis

import (
	"fmt"
	"git.168cad.top/zhengqiuyun/rym-util/conf"
	"git.168cad.top/zhengqiuyun/rym-util/const/YorN"
	"git.168cad.top/zhengqiuyun/rym-util/log"
	"github.com/garyburd/redigo/redis"
)

var pool *redis.Pool //创建redis连接池

func init() {
	pool = &redis.Pool{ //实例化一个连接池
		MaxIdle:     10,  //最初的连接数量
		MaxActive:   0,   //连接池最大连接数量,不确定可以用0（0表示自动定义），按需分配
		IdleTimeout: 300, //连接关闭时间 300秒 （300秒不使用自动关闭）
		Dial: func() (redis.Conn, error) { //要连接的redis数据库
			c, err := redis.Dial("tcp", conf.RedisAddr)
			if err != nil {
				return nil, err
			}
			if _, err := c.Do("AUTH", conf.RedisPwd); err != nil {
				c.Close()
				return nil, err
			}
			if _, err := c.Do("SELECT", conf.RedisDb); err != nil {
				c.Close()
				return nil, err
			}
			return c, nil
		},
	}
}

func Incr(key string) int64 {
	c := pool.Get() //从连接池，取一个链接
	defer c.Close() //函数运行结束 ，把连接放回连接池
	r, err := redis.Int64(c.Do("INCR", key))
	if err != nil {
		log.Error(err.Error())
	}
	return r
}

func IncrBy(key string, n int) int64 {
	c := pool.Get() //从连接池，取一个链接
	defer c.Close() //函数运行结束 ，把连接放回连接池
	r, err := redis.Int64(c.Do("INCRBY", key, n))
	if err != nil {
		log.Error(err.Error())
	}
	return r
}

func DecrBy(key string, n int) int64 {
	c := pool.Get() //从连接池，取一个链接
	defer c.Close() //函数运行结束 ，把连接放回连接池
	r, err := redis.Int64(c.Do("DECRBY", key, n))
	if err != nil {
		log.Error(err.Error())
	}
	return r
}

func Exists(key string) bool {
	c := pool.Get() //从连接池，取一个链接
	defer c.Close() //函数运行结束 ，把连接放回连接池
	r, err := redis.Int64(c.Do("EXISTS", key))
	if err != nil {
		return false
	}
	return r > 0
}

//SwitchOpen 在redis中存0或1,1代表OK
func SwitchOpen(key string) bool {
	r := GetString(key)
	log.Debug(fmt.Sprintf("缓存[%s]数据为：%s", key, r))
	return r == YorN.Y
}

func Get(key string) []byte {
	c := pool.Get() //从连接池，取一个链接
	defer c.Close() //函数运行结束 ，把连接放回连接池
	r, err := redis.Bytes(c.Do("GET", key))
	if err != nil {
		return nil
	}
	return r
}

func GetString(key string) string {
	c := pool.Get() //从连接池，取一个链接
	defer c.Close() //函数运行结束 ，把连接放回连接池
	r, err := redis.String(c.Do("GET", key))
	if err != nil {
		return ""
	}
	return r
}

func Set(k string, v string) {
	c := pool.Get() //从连接池，取一个链接
	defer c.Close() //函数运行结束 ，把连接放回连接池
	r, err := c.Do("SET", k, v)
	if err != nil {
		log.Error(err.Error())
	}
	log.Debug(fmt.Sprintf("设置缓存成功：%s", r))
}

func Expire(k string, t int) {
	c := pool.Get() //从连接池，取一个链接
	defer c.Close()
	r, err := c.Do("EXISTS", k, t)
	if err != nil {
		log.Error(err.Error())
	}
	log.Debug(fmt.Sprintf("设置缓存过期成功：%d", r))
}
